@@ -15,7 +15,6 @@
 
 #include "adw-avatar.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 
 #define NUMBER_OF_COLORS 14
 
@@ -45,8 +44,6 @@
  * ## CSS nodes
  *
  * `AdwAvatar` has a single CSS node with name `avatar`.
- *
- * Since: 1.0
  */
 
 struct _AdwAvatar
@@ -216,14 +213,54 @@ update_font_size (AdwAvatar *self)
 static void
 update_custom_image_snapshot (AdwAvatar *self)
 {
-  GtkSnapshot *snapshot = gtk_snapshot_new ();
+  GtkSnapshot *snapshot = NULL;
   GdkPaintable *square_image;
-  int height = gdk_paintable_get_intrinsic_height (self->custom_image_source);
-  int width = gdk_paintable_get_intrinsic_width (self->custom_image_source);
-  int size = MIN (width, height);
+  int width, height;
+  float scaled_width, scaled_height;
+  float size;
 
-  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - width) / 2.f, (size - height) / 2.f));
-  gdk_paintable_snapshot (self->custom_image_source, snapshot, width, height);
+  if (!self->custom_image_source)
+    return;
+
+  width = gdk_paintable_get_intrinsic_width (self->custom_image_source);
+  height = gdk_paintable_get_intrinsic_height (self->custom_image_source);
+
+  if (height == width && !GDK_IS_TEXTURE (self->custom_image_source)) {
+    gtk_image_set_from_paintable (self->custom_image, self->custom_image_source);
+    return;
+  }
+
+  size = self->size * gtk_widget_get_scale_factor (GTK_WIDGET (self));
+
+  if (width > height) {
+    scaled_height = size;
+    scaled_width = (float) width * scaled_height / (float) height;
+  } else if (width < height) {
+    scaled_width = self->size;
+    scaled_height = (float) height * scaled_width / (float) width;
+  } else {
+    scaled_width = scaled_height = size;
+  }
+
+  snapshot = gtk_snapshot_new ();
+  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT ((size - scaled_width) / 2.f, (size - scaled_height) / 2.f));
+
+  if (GDK_IS_TEXTURE (self->custom_image_source)) {
+    GskScalingFilter filter;
+
+    if (scaled_width > width || scaled_height > height)
+      filter = GSK_SCALING_FILTER_NEAREST;
+    else
+      filter = GSK_SCALING_FILTER_TRILINEAR;
+
+    gtk_snapshot_append_scaled_texture (snapshot,
+                                        GDK_TEXTURE (self->custom_image_source),
+                                        filter,
+                                        &GRAPHENE_RECT_INIT (0, 0, scaled_width, scaled_height));
+
+  } else {
+    gdk_paintable_snapshot (self->custom_image_source, snapshot, scaled_width, scaled_height);
+  }
 
   square_image = gtk_snapshot_free_to_paintable (snapshot, &GRAPHENE_SIZE_INIT (size, size));
   gtk_image_set_from_paintable (self->custom_image, square_image);
@@ -344,8 +381,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * The name of an icon to use as a fallback.
    *
    * If no name is set, `avatar-default-symbolic` will be used.
-   *
-   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -359,8 +394,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    *
    * It's only used to generate the color if [property@Avatar:show-initials] is
    * `FALSE`.
-   *
-   * Since: 1.0
    */
   props[PROP_TEXT] =
     g_param_spec_string ("text", NULL, NULL,
@@ -373,8 +406,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * Whether initials are used instead of an icon on the fallback avatar.
    *
    * See [property@Avatar:icon-name] for how to change the fallback icon.
-   *
-   * Since: 1.0
    */
   props[PROP_SHOW_INITIALS] =
     g_param_spec_boolean ("show-initials", NULL, NULL,
@@ -387,8 +418,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * A custom image paintable.
    *
    * Custom image is displayed instead of initials or icon.
-   *
-   * Since: 1.0
    */
   props[PROP_CUSTOM_IMAGE] =
     g_param_spec_object ("custom-image", NULL, NULL,
@@ -399,8 +428,6 @@ adw_avatar_class_init (AdwAvatarClass *klass)
    * AdwAvatar:size: (attributes org.gtk.Property.get=adw_avatar_get_size org.gtk.Property.set=adw_avatar_set_size)
    *
    * The size of the avatar.
-   *
-   * Since: 1.0
    */
   props[PROP_SIZE] =
     g_param_spec_int ("size", NULL, NULL,
@@ -432,7 +459,6 @@ adw_avatar_init (AdwAvatar *self)
   gtk_widget_set_parent (GTK_WIDGET (self->custom_image), self->gizmo);
 
   self->text = g_strdup ("");
-  self->size = -1;
 
   set_class_color (self);
   update_initials (self);
@@ -441,6 +467,7 @@ adw_avatar_init (AdwAvatar *self)
   update_visibility (self);
 
   g_signal_connect (self, "notify::root", G_CALLBACK (update_font_size), NULL);
+  g_signal_connect (self, "notify::scale-factor", G_CALLBACK (update_custom_image_snapshot), NULL);
 }
 
 /**
@@ -452,8 +479,6 @@ adw_avatar_init (AdwAvatar *self)
  * Creates a new `AdwAvatar`.
  *
  * Returns: the newly created `AdwAvatar`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_avatar_new (int         size,
@@ -474,8 +499,6 @@ adw_avatar_new (int         size,
  * Gets the name of an icon to use as a fallback.
  *
  * Returns: (nullable): the icon name
- *
- * Since: 1.0
  */
 const char *
 adw_avatar_get_icon_name (AdwAvatar *self)
@@ -493,8 +516,6 @@ adw_avatar_get_icon_name (AdwAvatar *self)
  * Sets the name of an icon to use as a fallback.
  *
  * If no name is set, `avatar-default-symbolic` will be used.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_icon_name (AdwAvatar  *self,
@@ -521,8 +542,6 @@ adw_avatar_set_icon_name (AdwAvatar  *self,
  *
  * Returns: (nullable): the text used to generate the fallback initials and
  *   color
- *
- * Since: 1.0
  */
 const char *
 adw_avatar_get_text (AdwAvatar *self)
@@ -541,8 +560,6 @@ adw_avatar_get_text (AdwAvatar *self)
  *
  * It's only used to generate the color if [property@Avatar:show-initials] is
  * `FALSE`.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_text (AdwAvatar  *self,
@@ -572,8 +589,6 @@ adw_avatar_set_text (AdwAvatar  *self,
  * Gets whether initials are used instead of an icon on the fallback avatar.
  *
  * Returns: whether initials are used instead of an icon as fallback
- *
- * Since: 1.0
  */
 gboolean
 adw_avatar_get_show_initials (AdwAvatar *self)
@@ -591,8 +606,6 @@ adw_avatar_get_show_initials (AdwAvatar *self)
  * Sets whether to use initials instead of an icon on the fallback avatar.
  *
  * See [property@Avatar:icon-name] for how to change the fallback icon.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_show_initials (AdwAvatar *self,
@@ -619,8 +632,6 @@ adw_avatar_set_show_initials (AdwAvatar *self,
  * Gets the custom image paintable.
  *
  * Returns: (nullable) (transfer none): the custom image
- *
- * Since: 1.0
  */
 GdkPaintable *
 adw_avatar_get_custom_image (AdwAvatar *self)
@@ -638,8 +649,6 @@ adw_avatar_get_custom_image (AdwAvatar *self)
  * Sets the custom image paintable.
  *
  * Custom image is displayed instead of initials or icon.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_custom_image (AdwAvatar    *self,
@@ -662,10 +671,9 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
     int height = gdk_paintable_get_intrinsic_height (custom_image);
     int width = gdk_paintable_get_intrinsic_width (custom_image);
 
-    if (height == width) {
-      gtk_image_set_from_paintable (self->custom_image, custom_image);
-    } else {
-      update_custom_image_snapshot (self);
+    update_custom_image_snapshot (self);
+
+    if (height != width && !GDK_IS_TEXTURE (custom_image)) {
       g_signal_connect_swapped (custom_image, "invalidate-contents",
                                 G_CALLBACK (update_custom_image_snapshot),
                                 self);
@@ -690,8 +698,6 @@ adw_avatar_set_custom_image (AdwAvatar    *self,
  * Gets the size of the avatar.
  *
  * Returns: the size of the avatar
- *
- * Since: 1.0
  */
 int
 adw_avatar_get_size (AdwAvatar *self)
@@ -707,8 +713,6 @@ adw_avatar_get_size (AdwAvatar *self)
  * @size: The size of the avatar
  *
  * Sets the size of the avatar.
- *
- * Since: 1.0
  */
 void
 adw_avatar_set_size (AdwAvatar *self,
@@ -731,6 +735,7 @@ adw_avatar_set_size (AdwAvatar *self,
     gtk_widget_remove_css_class (self->gizmo, "contrasted");
 
   update_font_size (self);
+  update_custom_image_snapshot (self);
 
   gtk_widget_queue_resize (GTK_WIDGET (self));
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SIZE]);
@@ -746,8 +751,6 @@ adw_avatar_set_size (AdwAvatar *self,
  * This can be used to export the fallback avatar.
  *
  * Returns: (transfer full): the texture
- *
- * Since: 1.0
  */
 GdkTexture *
 adw_avatar_draw_to_texture (AdwAvatar *self,
